/* Examples and Documentation for
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "descriptor.h"
#include "frechet_distance.h"
#include "hausdorff_distance.h"
#include "hausdorff_distance_man.h"
#include "hausdorff_distance_min.h"

#include <rtl/UILib.h>
#include <rtl/SolverLib.h>
#include <utils/descriptor_utils.h>

#include <array>

/*
 * Descriptor blok Frechetovy vzdlensti.
 */
namespace frechet_distance {

	scgms::TMetric_Descriptor descriptor = {
		id,
		L"Frechet distance"
	};
}

/*
 * Descriptor blok Hausdorffovy vzdalenosti.
 */
namespace hausdorff_distance {

	scgms::TMetric_Descriptor descriptor = {
		id,
		L"Hausdorff distance"
	};
}

/*
 * Descriptor blok Hausdorffovy vzdalenosti modifikovane Manhattanskou metrikou.
 */
namespace hausdorff_distance_man {

	scgms::TMetric_Descriptor descriptor = {
		id,
		L"Hausdorff distance using Manhattan"
	};
}

/*
 * Descriptor blok Hausdorffovy vzdalenosti modifikovane Minkowskiho normou.
 */
namespace hausdorff_distance_min {

	scgms::TMetric_Descriptor descriptor = {
		id,
		L"Hausdorff distance using Minkowski"
	};
}

/*
 * Pole dostupnych descriptoru metrik.
 */
const std::array<scgms::TMetric_Descriptor, 4> metric_descriptors = { { frechet_distance::descriptor, hausdorff_distance::descriptor, hausdorff_distance_man::descriptor, hausdorff_distance_min::descriptor}};

/*
 * Impelmentace rozhrani knihovny metrik pro vraceni metriky.
 */
HRESULT IfaceCalling do_get_metric_descriptors(scgms::TMetric_Descriptor const **begin, scgms::TMetric_Descriptor const **end) {

	return do_get_descriptors(metric_descriptors, begin, end);
}

/*
 * Impelmentace rozhrani knihovny metrik pro vytvoreni metriky.
 */
HRESULT IfaceCalling do_create_metric(const scgms::TMetric_Parameters *parameters, scgms::IMetric **metric) {

	// Frechetova vzdalenost.
	if (parameters->metric_id == frechet_distance::id)
		return Manufacture_Object<Frechet_Distance>(metric, *parameters);

	// Hausdorffova vzdalenost.
	if (parameters->metric_id == hausdorff_distance::id)
		return Manufacture_Object<Hausdorff_Distance>(metric, *parameters);

	// Hausdorffova vzdalenost modifikovane Manhattanskou metrikou.
	if (parameters->metric_id == hausdorff_distance_man::id)
		return Manufacture_Object<Hausdorff_Distance_Manhattan>(metric, *parameters);

	// Hausdorffova vzdalenost modifikovane Minkowskiho normou.
	if (parameters->metric_id == hausdorff_distance_min::id)
		return Manufacture_Object<Hausdorff_Distance_Minkowski>(metric, *parameters);

	return E_NOTIMPL;
}
